package de.hawhamburg.reachability.sensor.adapter;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.sensor.Sensor;
import de.hawhamburg.reachability.sensor.SensorAdapter;
import de.hawhamburg.reachability.sensor.SensorAdapterFactory;
import de.hawhamburg.reachability.sensor.SensorAgent;
import de.hawhamburg.reachability.sensor.SensorData;
import de.hawhamburg.reachability.sensor.adapter.json.JsonExampleMessage;
import de.hawhamburg.reachability.util.ConnectionUtil;
import de.hawhamburg.reachability.util.OutputUtil;

/**
 * Example {@link SensorAdapter} to illustrate the basic sensor agent implementation.<br><br>
 * 
 * This sensor adapter may provide some context information about the environment and/or its inhabitant. The measured
 * data represents the context dimension "example". It receives the data from a non existing sensor via the ActiveMQ. 
 * The data from the data source is frequently pushed onto its result topic, so no requests will be necessary.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class ExampleAdapter extends    Sensor
                            implements SensorAdapter  {
        
    /**
     * The constructor only delegates the supplied {@link ReasonerDimension} ("example") to the constructor of the
     * abstract {@link Sensor} class.<br><br>
     * 
     * The constructor of {@link Sensor} loads the local XML configuration from "/resource/sensor/ExampleAdapter.xml" 
     * and opens all required ActiveMQ connections.
     * 
     * @param dimension The {@link ReasonerDimension} for which this sensor is configured.
     */
    public ExampleAdapter(final ReasonerDimension dimension) {
        super(dimension);
    }

    /**
     * Retrieves the current inhabitants stress level via ActiveMQ and returns it as 
     * {@link SensorData}.
     * 
     * @return          The actual measured value as {@link SensorData} object. 
     */
    public final SensorData getSensorOutput() {
        // Get new data from the sensors data source.
        String json = ConnectionUtil.getMessageOnTopic(this.getResultTopic());
        
        // Fetch configuration variable.
        String test = this.getLocalConf("test");
        
        // If there are some...
        if (json != null && "true".equals(test)) {
            
            // Define a default value.
            double value = 0.0;
            
            // Transform JSON-string into an object.
            Gson g = new Gson();
            JsonExampleMessage msgObj = g.fromJson(json, JsonExampleMessage.class);
            
            // Transform the value into a double value.
            value = (double) msgObj.Value.ordinal();
            OutputUtil.debug("Received example value: " + msgObj.Value + " (Interpreted as " + value + ")");
            
            // Return a new sensor data object containing the simple class name, the context dimension and the value.
            return new SensorData(this.getClass().getSimpleName(), this.getDataDimension(), value);
        } else {
            // No data found...
            return null;
        }
    }

    /**
     * Main method which starts this {@link ExampleAdapter}.
     * 
     * @param args      The start arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers for a clear console.
        Logger.getRootLogger().removeAllAppenders();
        
        // Build this sensor adapter by using the SensorAdapterFactory.
        SensorAdapter sa = SensorAdapterFactory.getConnectedSensorAdapter(ExampleAdapter.class.getSimpleName(),
                                                                          ReasonerDimension.example);
        
        // Start the sensor adapter as sensor agent, which supplies the required control functions.
        new SensorAgent(sa);
    }
}
