package de.hawhamburg.reachability.tools.googleCalendar;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import com.google.gdata.client.Query;
import com.google.gdata.client.calendar.CalendarQuery;
import com.google.gdata.client.calendar.CalendarService;
import com.google.gdata.data.DateTime;
import com.google.gdata.data.IFeed;
import com.google.gdata.data.calendar.CalendarEventEntry;
import com.google.gdata.data.calendar.CalendarEventFeed;
import com.google.gdata.data.calendar.CalendarFeed;
import com.google.gdata.util.AuthenticationException;
import com.google.gdata.util.ServiceException;

import de.hawhamburg.reachability.util.OutputUtil;

/**
 * Adapter for the google calendar web-service.<br><br>
 * 
 * This class serves as adapter to the google calendar web-service. It provides a connection to the webservice using the
 * google gdata library. Furthermore it provides methods to fetch events from a certain calendar and to customize the
 * request by using certain user defined conditions.
 *
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class GoogleCalendarHook {
    
    /**
     * The google calendar object.
     */
    private CalendarService service = null;
    
    /**
     * Initializes a new {@link GoogleCalendarHook} with a supplied user and password. The connection to the google
     * calendar will be established using these informations. The communication to the google calendar web-service will
     * be done using an SSL connection.
     * 
     * @param user                  The users id as {@link String}.
     * @param password              The users password as {@link String}.
     */
    public GoogleCalendarHook(final String user,
                              final String password) {
        service = this.connect("de-hawhamburg-reachability-cService-1", user, password);
    }
    
    /**
     * Connects to the google calendar web-service using the supplied application name, user and password. The 
     * connection to the google calendar will be established using these informations
     * 
     * @param app                   The name of this application as {@link String}.
     * @param user                  The users id as {@link String}.
     * @param password              The users password as {@link String}.
     * @return                      The connection to the google calendar web-service as {@link CalendarService}.
     */
    private CalendarService connect(final String app,
                                    final String user,
                                    final String password) {
        CalendarService client = new CalendarService(app);
        try {
            client.setUserCredentials(user, password);
            client.useSsl();
        } catch (AuthenticationException e) {
            OutputUtil.error("[AuthenticationException]" + e.getMessage());
            client = null;
        }
        return client;
    }
    
    /**
     * Determines if this object is connected to the google calendar web-service.
     * 
     * @return                      <code>true</code> if this object is connected, otherwise <code>false</code>.
     */
    public final boolean isConnected() {
        return service != null;
    }
    
    /**
     * Fetches the accessible calendars within the established connection.
     * 
     * @return                      A {@link List} of calendars as {@link String}, which are accessible within the 
     *                              established connection.
     */
    public final List<String> getCalendars() {
        List<String> calendars = new ArrayList<String>();
        try {
            URL feedUrl = new URL("https://www.google.com/calendar/feeds/default/owncalendars/full");
            CalendarFeed resultFeed = this.getFeed(feedUrl, CalendarFeed.class);

            for (int i = 0; i < resultFeed.getEntries().size(); i++) {
                calendars.add(resultFeed.getEntries().get(i).getTitle().getPlainText());
            }
        } catch (MalformedURLException e) {
            OutputUtil.error("[MalformedURLException]" + e.getMessage());
        }
        return calendars;
    }
    
    /**
     * Fetches entries from a certain calendar within the established connection using supplied request conditions.
     * 
     * @param calendar              The calendars name as {@link String} for which the requests was launched.
     * @param amount                The maximum amount of results, <code>-1</code> is used as wildcard for all.
     * @param min                   The start time of the search as {@link DateTime}, <code>null</code> if indifferent.
     * @param max                   The end time of the search as {@link DateTime}, <code>null</code> if indifferent.
     * @param sortOrder             The sorting order as {@link String} (<code>ascending</code> or 
     *                              <code>descending</code>).
     * @return                      A {@link List} of {@link CalendarEventEntry} as search result.
     */
    public final List<CalendarEventEntry> getEvents(final String calendar,
                                                    final int amount,
                                                    final DateTime min,
                                                    final DateTime max,
                                                    final String sortOrder) {
        List<CalendarEventEntry> events = new ArrayList<CalendarEventEntry>();
        URL feedUrl;
        try {
            feedUrl = new URL("https://www.google.com/calendar/feeds/" + calendar + "/private/full");
            CalendarQuery query = new CalendarQuery(feedUrl);
            if (min != null) {
                query.setMinimumStartTime(min);
            }
            if (max != null) {
                query.setMaximumStartTime(max);
            }
            query.setStringCustomParameter("orderby", "starttime");
            if (sortOrder == null) {
                query.setStringCustomParameter("sortorder", "ascending");
            } else {
                query.setStringCustomParameter("sortorder", sortOrder);
            }
                
            CalendarEventFeed eventFeed = this.getFeed(query, CalendarEventFeed.class);
            
            if (eventFeed != null) {
                for (int x = 0; x < eventFeed.getEntries().size(); x++) {
                    if (x < amount || amount == -1) {
                        events.add(eventFeed.getEntries().get(x));
                    }
                }
            }
        } catch (MalformedURLException e) {
            OutputUtil.error("[MalformedURLException]" + e.getMessage());
        }
        return events;
    }

    /**
     * Fetches a feed by URL.
     * 
     * @param <T>                   The result feeds object type implementing {@link IFeed}.
     * @param url                   The URL to the feed.
     * @param feedClass             The result feeds object class.
     * @return                      The feed as <i>feedClass</i>.
     */
    private <T extends IFeed> T getFeed(final URL url,
                                        final Class<T> feedClass) {
        CalendarQuery query = new CalendarQuery(url);
        return this.getFeed(query, feedClass);
    }
    
    /**
     * Fetches a feed by query.
     * 
     * @param <T>                   The result feeds object type implementing {@link IFeed}.
     * @param query                 The query to the feed.
     * @param feedClass             The result feeds object class.
     * @return                      The feed as <i>feedClass</i>.
     */
    private <T extends IFeed> T getFeed(final Query query,
                                        final Class<T> feedClass) {
        if (this.service == null) {
            OutputUtil.error("[getFeed] Not connected to google calendar");
            return null;
        }
        try {
            return service.getFeed(query, feedClass);
        } catch (IOException e) {
            OutputUtil.error("[IOException]" + e.getMessage());
        } catch (ServiceException e) {
            OutputUtil.error("[ServiceException]" + e.getMessage());
        }
        return null;
    }
}
